#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2003 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# Apache Combined module for log reporting system


# Sets analysis parameters - specific for each module

use strict;
use FindBin qw($Bin);
use lib "$Bin/util/perl";
use Logrep::Common;

do "$Bin/logrep-common1.pl";

our (
	@dim, $from, $user, $authuser, $day, $monthmmm, $hour, $minute, $request, 
	$result, $size,	$method, $document, $kbsize, $month, $startdate, $enddate,
	$n, $logfilename, $dieopenmsg, $dieclosemsg, %monthnr, $config, $referrer,
	$useragent
	);
		
@dim = ( "from", "user", "authuser", "day", "month", "hour", "method", "document", "result", "kbsize", "referrer", "useragent");

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = 0;

ProcessApacheCombinedLogFile ($logfilename);

GenerateReport ("APACHE Combined Log file analysis",	{} );

###############
# SUBROUTINES #
###############

# ProcessApacheCombinedLogFile : 
#	Processes logfile in Combined format and populates counter arrays
#	Parameter : filename
#
sub ProcessApacheCombinedLogFile {

	my $logfile = shift;

	open LOGFILE, $logfile or die $dieopenmsg;

LOOP: 
	while (<LOGFILE>) {

		next if /^$/; # Skip empty lines

		my $line = $_;

		($from, $user, $authuser, $day, $monthmmm, $hour, $minute, $request, $result, $size, $referrer, $useragent) = 
			($line =~ m!(\S+) (\S+) (\S+) \[(\d\d)\/(\w\w\w)\/\d\d\d\d:(\d\d):(\d\d).+"(.+)" (\d+) (\S+) "(.*)" "(.*)"!) ;

		# parse request field
		($method, $document) = ($request ne "-") ? split (/ /, $request) : ("-", "-");
		
		# Some normalization
		($document) = split (/\?/, $document); # document w/o parameters
		$kbsize = int ($size / 1024); # Size in KB
		$month = $monthnr {$monthmmm}; # Month number
		
		($startdate = "$month/$day $hour:$minute") if ($. == 1);
				
		#DEBUG! print "$from, $user, $authuser, $day, $month, $hour, $minute, $method, $document, $result, $kbsize, $referrer, $useragent\n";
	
		PrepareAttributesForAnalyse ();
		
		GenerateTables (1, $size);
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}

