#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2004 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# Cisco VPN Concentrator module for log reporting system


use strict;
use integer;
use FindBin qw($Bin);
use lib "$Bin/util/perl";
use Logrep::Common;

do "$Bin/logrep-common1.pl";

# Variable declarations
our (
	@dim, $n,$dieclosemsg, $dieopenmsg, $nproc,
	$month, $day, $hour, $minute, $monthmmm, $code, $message, $from, $group, $user, $length
);

# Sets analysis parameters - specific for each module

@dim = ( 
	"month", "day", "hour", "code", "message", "from", "group", "user", "length" 
);

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = $nproc = 0;

ProcessCiscoVpnFile ($logfilename);

GenerateReport (
	"Cisco VPN Concentrator Log file analysis",
	{
		"Processed lines: " => $nproc,

	}
);

###############
# SUBROUTINES #
###############

# ProcessCiscoVpnFile : 
#	Processes Cisco VPN Concentrator logfile and populates counter arrays
#	Parameter : filename
#
sub ProcessCiscoVpnFile {

	open LOGFILE, shift or die $dieopenmsg;
	
	my $line = "";	
	my $i  = 0;

LOOP: 
	while (<LOGFILE>) {

		next if /^$/; # Skip empty lines

		$nproc++;
	
		$line = $_;

		($monthmmm, $day, $hour, $minute, $code, $message) =

		($line =~ /\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+\S+\s+(\S+)\s+(\S+)\s+(\d\d):(\d\d).+%(\S+): \S+: (.*)/);

		# Some normalization
		$month = $monthnr {$monthmmm}; # Month number

		# Remove some variable stuff from message

		$message =~ s/\(msgid=\S+\)//;	# The first (msgid=????????)

		$message =~ s/\(seq number 0x\S+\)//;	# (seq number 0x1372a8af)

		if ($message =~ /^\d+\.\d+\.\d+\.\d+:/) { # 192.168.253.1: xxxxxxxxxxxxxx
			($from, $message) = ($message =~ /^(\d+\.\d+\.\d+\.\d+):(.*)/);	
		}

		if ($message =~ /Group \[/) { # Group [xxxxxx] xxxxxxxxx
			($group, $message) = ($message =~ /Group \[(\S+)\] (.*)/);	
		}

		if ($message =~ /User \[/) { # User [xxxxxxx] xxxxxxx 
			($user, $message) = ($message =~ /User \[(\S+)\] (.*)/);	
		}
 
		if ($message =~ /total length :/) { # Message total length 
			($message, $length) = ($message =~ /(.+) : (\S+)/);
			$message =~ s/total length//;
		}
	
		($startdate = "$month/$day $hour:$minute") if ($nproc == 1);

		#DEBUG! print "$month, $day, $hour, $code, $message, $from\n";

		PrepareAttributesForAnalyse ();

		GenerateTables (1, 0);
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}
