#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2003 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# Checkpoint Firewall-1 NG module for log reporting system
 
# Sets analysis parameters - specific for each module
use strict;
use FindBin qw($Bin);
use lib "$Bin/util/perl";
use Logrep::Common;

do "$Bin/logrep-common1.pl";

our (
	@dim, $n, $logfilename, $dieopenmsg, $dieclosemsg,
	$day, $monthmmm, $hour, $minute, $origin, $type, $action, $interface, 
	$protocol, $source, $destination, $service, $rule, $origin, $sourceport,
	$month, %monthnr, $information, $user, $enddate, $startdate
	);
	
@dim = ( "month", "day", "hour", "origin", "type", "action", "interface", "protocol", "source", "destination", "service", "rule", "information", "user", "sourceport");

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = 0;

ProcessFirewall1NGLogFile ($logfilename);

GenerateReport (
	"Checkpoint Firewall-1 NG Logfile analysis", {}
);

###############
# SUBROUTINES #
###############

# ProcessFirewall1NGLogFile : 
#	Processes Firewall-1 export logfile and populates counter arrays
#	Parameter : Firewall-1 export log filename
#
sub ProcessFirewall1NGLogFile {

	my $logfile = shift;
		
	open LOGFILE, $logfile or die $dieopenmsg;

	<LOGFILE>; # Skip over first header line

LOOP: 
	while (<LOGFILE>) {

		next if /^$/; # Skip empty lines
		
		my $line = $_;

		($day, $monthmmm, $hour, $minute, $interface, $origin, $type, $action, $service, $source, $destination, $protocol, $rule, $sourceport, $user, $information) = 
			($line =~ /"\d+" "(\d+)(\w\w\w)\d+" "([^:]+):(\d\d):\d\d" ".*" "(.*)" "(.*)" "(.*)" "(.*)" "(.*)" "(.*)" "(.*)" "(.*)" "(.*)" "(.*)" "(.*)" "(.*)"/);

		# Some normalization
		$month = $monthnr {$monthmmm};
		
		($startdate = "$month/$day $hour:$minute") if ($. == 2); # first line was a header!

		#DEBUG! print "$day, $monthmmm, $hour, $minute, $interface, $origin, $type, $action, $service, $source, $destination,$protocol, $rule, $sourceport, $user, $information\n";
	
		PrepareAttributesForAnalyse ();

		GenerateTables (1, 0);
						
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}

