#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2003 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# Cisco Pix module for log reporting system
# v1.0 - Initial version, Nov 2002, Tevfik K. (tevfik@itefix.no)

# Sets analysis parameters - specific for each module
use strict;
use FindBin qw($Bin);
use lib "$Bin/util/perl";
use Logrep::Common;

do "$Bin/logrep-common1.pl";

our (
	@dim, $n, @sevcount, $logfilename, $dieopenmsg, $dieclosemsg,
	$monthmmm, $day, $hour, $minute, $pixnode, $severity, $code, $message,
	$month, %monthnr, $source, $sourceport, $destination, $destport, $flags, $event, $msghead, $msgend,
	$enddate, $startdate
	);
	
@dim = (

	"month", "day", "hour", "pixnode", "severity", "source", "sourceport", "destination", "destport", "event"
);

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = 0;

@sevcount = (0, 0, 0, 0, 0, 0, 0, 0);

ProcessPixFile ($logfilename);

GenerateReport (
	"Cisco PIX message analysis",
	{
		"(Alert, Critical, Error, Warning, Notification, Informational, Debugging) =" => "($sevcount[1], $sevcount[2], $sevcount[3], $sevcount[4],  $sevcount[5], $sevcount[6], $sevcount[7])"
	}
);

###############
# SUBROUTINES #
###############

# ProcessPixFile : 
#	Processes Pix logfile and populates counter arrays
#	Parameter : filename
#
sub ProcessPixFile {

	my $logfile = shift;
	
	open LOGFILE, $logfile or die $dieopenmsg;

LOOP: 
	while (<LOGFILE>) {

		next if /^$/; # Skip empty lines

		next if !/%PIX/; # Only Pix lines !!

		my $line = $_;

		($monthmmm, $day, $hour, $minute, $pixnode, $severity, $code, $message ) =		 
			($line =~ m!(\S+)\s+(\d+)\s+(\d\d):(\d\d):\d\d\s+(\S*).+%PIX-(\d)-(\d+):(.*)!) ;

		# Some normalization
		$month = $monthnr {$monthmmm}; # Month number
		$source = $sourceport = $destination = $destport = $flags = $event = "";
		
		$msghead = $msgend = "";
		
		($sevcount[$severity]++) if ($severity >= 1 && $severity <= 7);
		
	MSGPARSE: {
        	if (($code == 106001) || ($code == 106006) || ($code == 106007) ) {
        		($msghead, $source, $sourceport, $destination, $destport, $msgend) = 
        			($message =~ m!(.*) from (\S+)/(\S+) to (\S+)/(\S+) (.*)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}
 
        	if ($code == 106002) {
        		($msghead, $source, $sourceport, $destination, $destport) = 
        			($message =~ m!(.*) src (\S+) (\S+) dest (\S+) (\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}
        	
        	if ($code == 106003) {
        		($msghead, $source, $destination, $msgend) = 
        			($message =~ m!(.*) src (\S+) dest (\S+) (.*)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}
        	
        	if ($code == 106009) {
        		($msghead, $source, $destination, $destport, $msgend, $sourceport) = 
        			($message =~ m!(.*) for (\S+) to (\S+)/(\S+) (\S+ \S+ \S+ \S+ \S+ \S+) (\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

		if ($code == 106010) {
        		($msghead, $source, $sourceport, $destination, $destport) = 
        			($message =~ m!(Deny inbound \S+) src \S+:(\S+)/(\S+) dst \S+:(\S+)/(\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
		}

		if ($code == 106012) {
        		($msghead, $source, $destination, $msgend) = 
        			($message =~ m!(.*) from (\S+) to (\S+)(.*)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

		if ($code == 106013) {
        		($msghead, $source, $destination, $msgend) = 
        			($message =~ m!(.*) from (\S+) to PAT address (\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

        	if ($code == 106016) {
        		($msghead, $source, $destination, $msgend) = 
        			($message =~ m!(.*) from (\S+) to (\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

		if ($code == 106017) {
        		($msghead, $source, $destination, $msgend) = 
        			($message =~ m!(.*) src (\S+) dest to (\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

		if ($code == 106018) {
        		($msghead, $source, $destination, $msgend) = 
        			($message =~ m!(.*) src (\S+) dest (\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

		if ($code == 106023) {
        		($msghead, $source, $sourceport, $destination, $destport, $msgend) = 
        			($message =~ m!(Deny \S+) src \S+:(\S+)/(\S+) dst \S+:(\S+)/(\S+) (.*)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
		}

        	if ($code == 108002) {
        		($msghead, $source) = 
        			($message =~ m!(.*) out (\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

        	if ($code == 109011) {
        		($msghead) = 
        			($message =~ m!(.*) user!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

		if ($code == 112001) {
        		($msgend) = 
        			($message =~ m!.* (pix .*)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

  		if ($code == 201003) {
        		($msghead) = 
        			($message =~ m!(.*) for!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

  		if ($code == 304007) {
        		($msghead, $source, $msgend) = 
        			($message =~ m!(URL Server) (\S+) (.*)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

  		if ($code == 401004) {
        		($msghead, $source, $destination, $msgend) = 
        			($message =~ m!(\S+): (\S+) ==> (\S+) (.*)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

		# Notifications level - 5
		 if ($code == 109012) {
        		($msghead) = "Authen Session End";
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

		if ($code == 305005 || $code == 305006) {
        		($msghead, $source, $sourceport, $destination, $destport) = 
        			($message =~ m!(.* for \S+) src \S+:(\S+)/(\S+) dst \S+:(\S+)/(\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
		}

 		if ($code == 111001 || $code == 111007) {
        		($msghead, $source, $msgend) = 
        			($message =~ m!(.*): (\S+) (.*)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

 		if ($code == 111003 || $code == 111004 || $code == 111005) {
        		($source, $msgend) = 
        			($message =~ m!(\S+) (.*)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

 		if ($code == 111006) {
        		($msghead, $source) = 
        			($message =~ m!(.*) at (\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

		 if ($code == 111008) {
        		($msghead) = $message;
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

 		if ($code == 199001) {
        		($msghead, $source) = 
        			($message =~ m!(.*) from (\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

		if ($code == 304001) {
        		($source, $msghead, $destination) = 
                                ($message =~ m!(\S+) (Accessed .*URL) (\S+):!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
		}

		# Informational level - 6

        	if ($code == 106015 || $code == 109001 || $code == 109005 || 
			$code == 109006 || $code == 109007 || $code == 109008) {
        		($msghead, $source, $sourceport, $destination, $destport) = 
        			($message =~ m!(.*) from (\S+)/(\S+) to (\S+)/(\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

        	if ($code == 109002 || $code == 109003 || $code == 109009) {
        		($msghead, $source, $sourceport, $destination, $destport, $msgend) = 
        			($message =~ m!(.*) from (\S+)/(\S+) to (\S+)/(\S+) (.*)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

		if ($code == 199002 || $code == 199003 || $code == 199005) {
        		($msghead) = $message;
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

        	if ($code == 302001) {
        		($msghead, $source, $sourceport, $destination, $destport, $msgend) = 
        			($message =~ m!(Built \S+ \S+ connection) \S+ for faddr (\S+)/(\S+) gaddr \S+ laddr (\S+)/(\S+) (.*)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

        	if ($code == 302003) {
        		($msghead, $destination, $source, $sourceport) = 
        			($message =~ m!(.*) for faddr (\S+) laddr (\S+)/(\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

        	if ($code == 302004) {
        		($msghead, $source, $destination, $destport) = 
        			($message =~ m!(.*) for faddr (\S+) to laddr (\S+)/(\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

        	if ($code == 302005 || $code == 302006) {
        		($msghead, $destination, $destport, $source, $sourceport) = 
        			($message =~ m!(.*) for faddr (\S+)/(\S+) gaddr \S+ laddr (\S+)/(\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

		if ($code == 302013 || $code == 302015) {
        		($msghead, $source, $sourceport, $destination, $destport, $msgend) = 
        			($message =~ m!(Built \S+ \S+ connection) \S+ for \S+:(\S+)/(\S+) \S+ to \S+:(\S+)/(\S+) \S+(.*)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
		}

		if ($code == 302014 || $code == 302016) {
        		($msghead, $source, $sourceport, $destination, $destport) = 
        			($message =~ m!(Teardown \S+ connection) \S+ for \S+:(\S+)/(\S+) to \S+:(\S+)/(\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
		}

        	if ($code == 303002) {
        		($source, $msghead, $destination) = 
        			($message =~ m!(\S+) (\S+) (\S+):!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

       		if ($code == 303004) {
        		($msghead, $destination, $msgend) = 
        			($message =~ m!(URL Server) (\S+) (request failed)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

       		if ($code == 305001) {
        		($msghead, $destination, $destport, $source, $sourceport) = 
        			($message =~ m!(Portmapped translation built) for gaddr (\S+)/(\S+) laddr (\S+)/(\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

       		if ($code == 305002) {
        		($msghead, $destination, $source) = 
        			($message =~ m!(Translation built) for gaddr (\S+) to (\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

      		if ($code == 305003) {
        		($msghead, $destination, $source) = 
        			($message =~ m!(Teardown translation) for global (\S+) local (\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

      		if ($code == 305004) {
        		($msghead, $destination, $destport, $source, $sourceport) = 
        			($message =~ m!(Teardown portmap translation) for global (\S+)/(\S+) local (\S+)/(\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

      		if ($code == 305007) {
        		($msghead, $source, $msgend) = 
        			($message =~ m!.* (Orphan IP) (\S+) (.*)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

      		if ($code == 307001 || $code == 307002 || $code == 307003 || $code == 308001 || $code == 309002) {
        		($msghead, $source) = 
        			($message =~ m!(.*) from (\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

      		if ($code == 609001) {
        		($msghead, $source) = 
        			($message =~ m!(Built \S+) \S+:(\S+)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

		# Debugging level - 7

        	if ($code == 106011) {
        		($msghead) = "Deny inbound (no xlate)";
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

        	if ($code == 109014) {
        		($msghead) = "uauth_lookup_net fail for uauth_in()";
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

        	if ($code == 304005) {
         		($msghead, $source, $msgend) = 
        			($message =~ m!(URL Server) (\S+) (request pending URL)!);
        		$event = "$code: $msghead $msgend";
        	}

       		if ($code == 701001) {
        		($msghead) = "alloc_user() out of Tcp_user objects";
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

       		if ($code == 709001) {
        		($msghead, $msgend) = (
				$message =~ m!(\S+) (Rep CI ioctl)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}

		if ($code == 709002) {
        		($msghead, $msgend) = (
				$message =~ m!(\S+) (Rep no replication)!);
        		$event = "$code: $msghead $msgend";
        		last MSGPARSE;
        	}


		$event = "$code: unprocessed by logrep";
   
    		}
				
		($startdate = "$month/$day $hour:$minute") if ($. == 1);

		#DEBUG!	print "$month, $day, $hour, $minute, $pixnode, $severity, $source, $sourceport, $destination, $destport, $event\n";

		PrepareAttributesForAnalyse ();
		
		GenerateTables (1,0);
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}
