#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2004 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# SUS (Software Update Services) module for logrep


# Sets analysis parameters - specific for each module
use strict;
use FindBin qw($Bin);
use lib "$Bin/util/perl";
use Logrep::Common;

do "$Bin/logrep-common1.pl";

our (
	@dim, $n, $nl, $nstart, $logfilename, $dieopenmsg, $dieclosemsg,
	$month, $day, $hour, $minute, $from, $to, $client, $activity, $item, $platform, $language, $status, $error, $message,
	$enddate, $startdate, %lupclient, %lupactivity, %lupstatus, $susstr
	);
	
@dim = ( "month", "day", "hour", "from", "to", "client", "activity", "item", "platform", "language", "status", "error", "message");

# Lookup tables
%lupclient = (
	'AU' => "Download-Installation",
	'IU' => "Initialization",
	'IU_Site' => "WU web site"
);

%lupactivity = (
	'n' => "Init",
	's' => "Self-update",
	'd' => "Detection",
	'w' => "Download",
	'i' => "Installation"
);

%lupstatus = (
	's' => "Success",
	'r' => "Success-Reboot",
	'f' => "Failure",
	'c' => "Cancel",
	'd' => "Decline",
	'n' => "No items",
	'p' => "Pending"
);

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = 0;

ProcessSusLogFile ($logfilename);

GenerateReport (
	"SUS Log file analysis",
	{
		"Processed lines:" => $n,
		"SUS lines:" => $nl
	}
);

###############
# SUBROUTINES #
###############

# ProcessSusLogFile : 
#	Processes Sus logfile and populates counter arrays
#	Parameter : log filename
#
sub ProcessSusLogFile {

	my $logfile = shift;
	
	$nl = 0;
		
	open LOGFILE, $logfile or die $dieopenmsg;

LOOP: 
	while (<LOGFILE>) {

		next if not /wutrack\.bin/; # Only lines with wutrack.bin are interesting

		my $line = $_;
		
		$nl++;
		
		($month, $day, $hour, $minute, $from, $to, $susstr)= 
			($line =~ m"\d\d\d\d-(\d\d)-(\d\d) (\d\d):(\d\d)\S+ (\S+) \S+ (\S+) \S+ \S+ \S+ (\S+)") ;
		
		($client, $activity, $item, $platform, $language, $status, $error, $message) =
			($susstr =~ /\S+&\S+&C=(\S+)&A=(\S+)&I=(\S*)&\S+&P=(\S+)&L=(\S+)&S=(\S+)&E=(\S+)&M=(\S*)&/);

		# Lookup
		$client = $lupclient {$client} if exists $lupclient {$client};
		$activity = $lupactivity {$activity} if exists $lupactivity {$activity};
		$status = $lupstatus {$status} if exists $lupstatus {$status};
		
		($startdate = "$month/$day $hour:$minute") if ($nl == 1);

		#DEBUG! print "$month, $day, $hour, $minute, $from, $to, $client, $activity, $item, $platform, $language, $status, $error, $message\n";

		PrepareAttributesForAnalyse ();
		
		GenerateTables (1, 0);
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}
