#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2004 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# Watchguard module for log reporting system


use strict;
use integer;
use FindBin qw($Bin);
use lib "$Bin/util/perl";
use Logrep::Common;

do "$Bin/logrep-common1.pl";

# Variable declarations
our (
	@dim, $n,$dieclosemsg, $dieopenmsg, $nproc,
	$month, $day, $hour, $minute, $fw, $pri, $rule, $source, $destination, 
	$protocol, $srcport, $destport, $indev, $inport, $rc, $message
);

# Sets analysis parameters - specific for each module

@dim = ( 
	"month", "day", "hour", "fw", "pri", "rule", "source", "destination", "protocol", 
	"srcport", "destport", "indev", "inport", "rc", "message"
);

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = $nproc = 0;

ProcessWatchguardFile ($logfilename);

GenerateReport (
	"Watchguard Log file analysis",
	{
		"Processed lines: " => $nproc,

	}
);

###############
# SUBROUTINES #
###############

# ProcessWatchguardFile : 
#	Processes Watchguard logfile and populates counter arrays
#	Parameter : filename
#
sub ProcessWatchguardFile {

	open LOGFILE, shift or die $dieopenmsg;
	
	my $line = "";	
	my $i  = 0;

LOOP: 
	while (<LOGFILE>) {

		next if /^$/; # Skip empty lines

		next if !/id=firewall/; # Skip lines not containing 'id=firewall'
	
		$nproc++;
	
		$line = $_;

		($month, $day, $hour, $minute, $fw, $pri, $rule, $source, $destination, $protocol, $srcport, $destport, $indev, $inport, $rc, $message)	=
		($line =~ /.*time=\"\d\d\d\d-(\d\d)-(\d\d) (\d\d):(\d\d):.*fw=(\S+)\s+pri=(\S+)\s+rule=\"(.+)\"\s+src=(\S+)\s+dst=(\S+)\s+proto=(\S+)\s+sport=(\S+)\s+dport=(\S+)\s+indev=(\S+)\s+inport=(\S+)\s+rc=(\S+)\s+msg=\"(.*)\"/);  
	
		($startdate = "$month/$day $hour:$minute") if ($nproc == 1);

		#DEBUG! print "$month, $day, $hour, $fw, $pri, $rule, $source, $destination, $protocol, $srcport, $destport, $indev, $inport, $rc, $message\n";

		PrepareAttributesForAnalyse ();

		GenerateTables (1, 0);
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}
