#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# WinInstall log file module for log reporting system

# Sets analysis parameters - specific for each module
use strict;
use FindBin qw($Bin);
use lib "$Bin/util/perl";
use Logrep::Common;

do "$Bin/logrep-common1.pl";

our (
	@dim, $n, $nproc, $logfilename, $dieopenmsg, $dieclosemsg, $enddate, $startdate, $resultmsg,
	$month, $day, $hour, $minute, $package, $installtype, $result, $netaddr, $workstation, $user
	);

@dim = ( "month", "day", "hour", "installtype", "result", "package", "netaddr", "workstation", "user");

################
# MAIN PROGRAM #
################

# initialize counters
$n = $nproc = 0;

SetCommonParameters ();

ProcessWinInstallLogFile ($logfilename);

$resultmsg = sprintf "Processed lines: %d", $nproc;

GenerateReport ("WinInstall Log file analysis",

	{
		$resultmsg => "",
	}
);

###############
# SUBROUTINES #
###############

# ProcessWinInstallLogFile : 
#	Processes squid logfile and populates counter arrays
#	Parameter : WinInstall log filename
#
sub ProcessWinInstallLogFile {

	my $logfile = shift;

	open LOGFILE, $logfile or die $dieopenmsg;

LOOP: 
	while (<LOGFILE>) {

		next if /^$/; # Skip empty lines

		$nproc++;

		my $line = $_;

		($month, $day, $hour, $minute, $installtype, $result, $package, $netaddr, $workstation, $user) =

			($line =~ m|(\d\d)/(\d\d)/\S+\t(\d\d):(\d\d):\S+\t\S\t(.+)\t(.+): (.+)\tNetAddr:(\S+)\tWksName:(\S+)\tUserName:(\S+)|);

				
		($startdate = "$month/$day $hour:$minute") if ($nproc == 1);
		
		#DEBUG!	print "$month, $day, $hour, $minute, $installtype, $result, $package, $netaddr, $workstation, $user\n";

		PrepareAttributesForAnalyse ();
		
		GenerateTables (1, 0);
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}
